% function [x] = conjgradtrain(x0,lsfun,lsparams,ogfun,ogparams,varargin)
% x0 - initial value of the solution [d,1]
% lsfun - pointer to line search function
% function [alpha] = lsfun(x0,direction,ogfun,params)
%         x0 - location to start line search [d,1]
%         direction - direction from x0 along which to search [d,1]
%         ogfun - poitner to objective/gradient function
%         params - cell array of parameters to pass to ogfun
%         alpha - appx. location of minimum along direction
%                 appx. minimium is: x0+alpha.*direction
% lsparams - cell array of parameters to pass to lsparams
% ogfun - pointer to objective/gradient calculation function
% function [obj,grad] = ogfun(x,params)
%     x - parameter value at which to calculate obj/grad [d,1]
%     params - additional parameters
%     obj - objective value at x [scalar]
%     grad - gradient at x [d,1]
% ogparams - cell array of parameters to pass to ogfun
% x - final value of the solution (appxroximate local minimum) [d,1]
%
% Additional parameters given by name/value pairs via varagin.
% E.g. conjgrad(x0,lsfun,params,ogfun,'verbose',0)
%
% This is an revision (with nonnegative constraint) of  Polak-Ribiere Nonlinear Conjugate
% Gradients for optimal solution, as described by "An introduction to the conjugate
% gradient method without the agonizing pain" by Shewchuk (1994) and
% "Numerical Optimization" by Nocedal and Wright.
%


function [x,numiter] = conjgradtrain(x0,lsfun,lsparams,ogfun,ogparams,varargin)
  fn = mfilename;
  if nargin() < 5
    error('insufficient parameters')
  end

  % Parameters to be set via varargin
  verbose = 1; % print progress information if true
  tol = 1e-1; % geometric decrease in gradient magnitude to declare minimum
  maxiter = 1000; % stop after this many iterations (if no minimum found)
  nu = 0.1;
  abstol = 0.00001; % stop if gradient magnitude goes below this
  allowNonDecrease = 0; % don't stop if line search fails to find decrease
  digits = 12; % digits of precision to use for objective comparisons
  % Process varargin
  paramgt;

  t0 = clock;
  t1 = t0;
  ogcalls = 0;
  x = x0;
  x(x<0) = 0;
  numiter = 0;
  j = 0;
  alpha = 1e-10;

  ogcalls = ogcalls + 1;
  [obj,dx] = ogfun(x,ogparams{:});
  r = -dx;
  s = r;
  d = s;
  deltanew = full(r'*d);
  deltazero = deltanew;
 
  while numiter < maxiter & abs(deltanew) > tol*abs(deltazero) & abs(deltanew) > abstol
    numiter = numiter + 1;
    j = j + 1;
    prevobj = obj;
    if alpha < 1e-10
     alpha = 1e-10;
    end
    %checkgrad2(ogfun,x,ogparams);
    [alpha,obj,dx,ogc] = lsfun(x,obj,dx,d,ogfun,ogparams,'alpha0',alpha,'verbose',verbose,lsparams{:});
    if abs(obj-prevobj) < tol,
        break;
    end;
    ogcalls = ogcalls + ogc;
    x = x + alpha.*d;
    x(x < 0) = 0;
    r = -dx;
    deltaold = deltanew;
    deltamid = full(r'*s);
    deltanew = full(r'*r);
    beta = (deltanew-deltamid)./deltaold;
    d = r + max(0,beta).*d;
    if deltamid./deltanew >= nu | d'*dx >= 0
      if verbose >= 3 & length(x0) > 1
        fprintf(1,'RESET\n');
      end
      d = r;
      j = 0;
    end
    s = r;
  end
  if verbose
    fprintf(1,'%s i=%d delta=%1.3e obj=%1.3e  Time: %.1f s  Calls: %d\n',fn,numiter,deltanew,obj,etime(clock,t0),ogcalls);
  end

