% Quadratic interpolation line search with backtracking fallback;
% Uses Strong Wolfe termination conditions; returns distance along
% direction (alpha) and objective (obj) and gradient (dx) at
% x0+alpha.*direction
%
% function [alpha,obj,dx] = cgLineSearch(x0,obj0,dx0,direction,ogfun,ogparams,varargin)
% x0 - initial parameter value [d,1]
% obj0 - objective value at x0 [scalar]
% dx0 - gradient at x0 [d,1]
% direction - direction along which to find minimum [d,1]
% ogfun - pointer to objective/gradient calculation function
% function [obj,dx] = ogfun(x,params)
%     x - parameter value at which to calculate obj/grad [d,1]
%     params - additional parameters
%     obj - objective value at x [scalar]
%     dx - gradient at x [d,1]
% alpha - approximate minimum along direction is found at x0+alpha.*direction
%
% Additional parameters given by name/value pairs via varargin.
% E.g. cgLineSearch(w,obj0,dx0,d,objfn,'c2',1e-2,'verbose',0)
%
% Secant Line Search algorithm taken from "An introduction to the
% conjugate gradient method without the agonizing pain" by Jonathan
% Shewchuk (1994).
% Adapted by Zhenqiu Liu. UMB

function [alpha,obj,dx,ogcalls] = mycgLineSearch(x0,obj0,dx0,direction,ogfun,ogparams,varargin)
  if nargin < 6
    error('Insufficient parameters');
  end
  % Parameters that can be set via varargin
  seciter = 5; % maximum number of quadratic interpolation iterations
  alpha0 = 1e-8;
  c1 = 1e-4; % required decrease in objective (relative to gradient)
  c2 = 1e-1; % required decrease in directional derivative
  digits = 12; % digits of precision to use for objective comparisons
  gamma = 10;
  verbose = 1;
  % Process varargin
  paramgt;
  % Check initial values
  if alpha0 <= 0
    error('alpha0 must be greater than zero');
  end

  %%%%% Begin line search %%%%%
  %hessdiag = 0; % bogus variable, used in some subroutines shared with pccg
  obj = obj0;
  etazero = full(dx0'*direction);
  etaprev = etazero;
  alpha = alpha0;
  ogcalls = 0;
  %findNonZeroAlpha;
  xx = x0+alpha.*direction;
  xx(xx<0) =0;
  [obj,dx] = ogfun(xx,ogparams{:});
  ogcalls = ogcalls + 1;
  oldalpha = alpha;
  oldobj = obj;
  olddx = dx;

  % Make sure current step yields decrease in objective
 
  preAlpha = alpha;
  while (pround(obj,digits) > pround(obj0 + c1.*alpha.*etazero,digits)) && sum(x0+alpha.*direction ~= x0)>0
        if ogcalls > 1
           if oldalpha > alpha./gamma && oldobj < obj0 + c1.*oldalpha.*etazero
              alpha = oldalpha;
              obj = oldobj;
              dx = olddx;
    
              break;
           end
        end
        alpha = alpha ./ gamma;
        xx = x0+alpha.*direction;
        xx(xx < 0) = 0;
        [obj,dx] = ogfun(xx,ogparams{:});
         ogcalls = ogcalls + 1;
   end
  beta = alpha;
  eta = full(dx'*direction);
  i = 0;
  % Try to find a point with lower directional derivative
  % Stop if too many iterations, large objective, no change, or small alpha
  while abs(eta) > c2*abs(etazero) && i < seciter && pround(obj,digits) <= pround(obj0,digits) && etaprev ~= eta && sum(xx ~= x0)>0
    % Want newBeta/(eta-0) = oldBeta/(etaprev-eta)
    beta = eta.*beta./(etaprev-eta);
    oldalpha = alpha;
    oldobj = obj;
    olddx = dx;

    alpha = alpha + beta;
    % Negative alpha could be a sign that initial parameter vector is bad
    if alpha <= 0
        alpha = 1;
    end
    etaprev = eta;
    i = i + 1;
    xx = x0+alpha.*direction;
    xx(xx<0)=0;  % bring xx back to xx>=0
    [obj,dx] = ogfun(xx,ogparams{:});
    ogcalls = ogcalls + 1;
    eta = full(dx'*direction);
  end
  
 % Find nonzeroalpha. 
  if x0+alpha.*direction == x0
     preAlpha = alpha;
     while x0+alpha.*direction == x0 && sum(abs(alpha.*direction))>0  
           alpha = alpha .* gamma;
     end
     xx = x0+alpha.*direction;
     xx(xx < 0) = 0;
     [obj,dx] = ogfun(xx,ogparams{:});
     ogcalls = ogcalls + 1;
   end
  
  %backtrack;
 
  preAlpha = alpha;
  while (pround(obj,digits) > pround(obj0 + c1.*alpha.*etazero,digits)) && sum(x0+alpha.*direction ~= x0)>0
       if ogcalls > 1
          if oldalpha > alpha./gamma && oldobj < obj0 + c1.*oldalpha.*etazero
             alpha = oldalpha;
             obj = oldobj;
             dx = olddx;
             
             break;
          end
       end
       alpha = alpha ./ gamma;
       xx = x0+alpha.*direction;
       xx(xx < 0) = 0;
       [obj,dx] = ogfun(xx,ogparams{:});
       ogcalls = ogcalls + 1;
  end
  
 %checkConditions;
xx = x0+alpha.*direction;
xx(xx<0) = 0;
if sum(xx ~= x0)==0 & verbose 
  fprintf(1,'Warning: Line search yielded no change in position.\nMay have reached limit of precision.\n');
end

